##############################################################################
#
#  Setup script for the ESO Scisoft scientific software collection
#
#  This file should be sourced using the sh/ksh/bash dot "." command
#
#      . /scisoft/bin/Setup.bash
#
#  by all users of the software. They should be no need to do anything else.
#
#  If you encounter any problems, email scihelp@eso.org.
#
##############################################################################
#
#  Revision history
#
#  ??/11/2000 - kmaguire - created, based on Setup.csh
#  ??/01/2005 - sturolla - modified and cleaned up
#  07/06/2005 - ahuxley  - converted to .src with substitutable markers, 
#                          corrected eclipse and python root directory tests,
#                          stripped redundant curly braces and quotes, changed
#                          exits to returns.
#  
##############################################################################

#
#  echo'ing in scp sessions not allowed, so use this function instead of
#  echo.
#

echo_if_tty() { [ ! -t 1 ] || echo "$1"; }

#
#  Before doing anything check the shell
#

if [ "X$SHELL" != X ] && [ `basename $SHELL` != bash ]; then
    echo_if_tty "please start a bash shell before setting up scisoft"
    #  Don't use exit! It can potentially log the user out!
    return -1
fi

#
#  Find out which arch we are running on
#

if [ `uname` != Linux ]; then
    echo_if_tty "unsupported architecture"
    #  Don't use exit! It can potentially log the user out!
    return -1
fi

#
#  Add the main Scisoft binaries to the command search path
#

if [ "X$PATH" != X ]; then
    export PATH=/scisoft/bin:$PATH
else
    export PATH=/scisoft/bin
fi

#
#  Add the main Scisoft libraries to the library search path
#

if [ "X$LD_LIBRARY_PATH" != X ]; then
    export LD_LIBRARY_PATH=/scisoft/lib:$LD_LIBRARY_PATH
else
    export LD_LIBRARY_PATH=/scisoft/lib
fi

#
#  Add the main Scisoft manual pages to the manual page search path
#

if [ "X$MANPATH" != X ]; then
    export MANPATH=/scisoft/man:$MANPATH
else
    export MANPATH=/scisoft/man
fi

#
#  Add the Scisoft X application defaults directory to XUSERFILESEARCHPATH.
#

if [ "X$XUSERFILESEARCHPATH" != X ]; then
    export XUSERFILESEARCHPATH=/scisoft/lib/X11/app-defaults:$XUSERFILESEARCHPATH
else
    export XUSERFILESEARCHPATH=/scisoft/lib/X11/app-defaults
fi

#
#  IRAF related definitions
#

if [ -d /scisoft/share/iraf ]; then
    #  Note trailing slashes
    export iraf=/scisoft/share/iraf/iraf/
    export stsdas=/scisoft/share/iraf/extern/stsdas/
    export tables=/scisoft/share/iraf/extern/tables/
    if [ -f ${iraf}unix/hlib/irafuser.bash ]; then
        . ${iraf}unix/hlib/irafuser.bash
        #  The above script sets F77, which is quite naughty of it; 
        #  it can cause users' own compiles of totally unrelated software
        #  to fail.
        export F77_IRAF=$F77
        unset F77
    fi
    export IRAFARCH=redhat

    #  From: "Warren J. Hack" <hack@stsci.edu>
    #  To: Richard Hook <rhook@eso.org>
    #  CC: greenfield@stsci.edu
    #  Subject: Re: Greetings and a question
    #  
    #  Hello,
    #
    #  >I just had a funny problem on my new machine here and I thought I
    #  >would check if you had seen it before and had a simple answer. I have
    #  >a new Linux FC3 machine which has IRAF 2.12.2a (redhat). I also have
    #  >PyRAF 1.1.2 (2005Mar19)  and Python 2.4. The OS is
    #  >Fedora Core release 3 (Heidelberg). If I start the cl all seems well,
    #  >however, from Pyraf I get:
    #  >
    #  >--> imstat dev$pix
    #  >#               IMAGE      NPIX      MEAN    STDDEV       MIN       MAX
    #  >             dev$pix    262144     108.3     131.3       -1.    19936.
    #  >
    #  >PANIC in `/scisoft/share/iraf/iraf/bin.redhat/x_images.e': Memory has
    #  >been corrupted
    #  >Killing IRAF task `imstatistics'
    #  >Traceback (innermost last):
    #  > File "<CL script CL1>", line 1, in ?
    #  >   iraf.imstat('dev$pix', _save=1)
    #  >IrafError: Error running IRAF task imstatistics
    #  >('Not a legal IRAF pipe record', 32, 'Broken pipe')
    #  >--> .exit
    #  >
    #  >which is very strange, as this is exactly the same IRAF (as far as I
    #  >can see) to the one I get from the CL.
    #  >
    #  >I half suspect this is an environment rather than fundamental problem,
    #  >but don't know where to look.
    #  >
    #  >Any ideas?
    #  >
    #  >Cheers, Richard
    #
    #  The error you report appears similar to one that Nadia recognized for
    #  another help call.  It is a known IRAF problem on some platforms with
    #  the solution being to add the following line to your ~/.login file (or
    #  whatever unix login file serves the same purpose):
    #  
    #  limit stacksize unlimited
    #  
    #  This should correct the problem.  In the meantime, I hope it cools off
    #  for you soon, or at the very least, that the humidity moves out.
    #  
    #  Cheers,
    #  Warren
    ulimit -s unlimited

fi

#
#  IDL related definitions
#

if [ -d /scisoft/share/idl ]; then
    #  Marco Lombardi sorted out Harald Kuntschner's problem with
    #  strsplit() which was because libraries in /scisoft/share/idl/idl/lib
    #  are not included in IDL_PATH. Actually, rather than including a long
    #  list of specific subdirectories of this, then we just include the whole
    #  tree.
    export IDL_PATH="<IDL_DEFAULT>:+/scisoft/share/idl/astron:+/scisoft/share/idl/jhuapl:+/scisoft/share/idl/starfinder:+/scisoft/share/idl/atv"
    export IDL_DIR=/scisoft/share/idl/idl

   
    #  Marco wants some settings related to the display for ESO only.
    if [ -f /scisoft/share/idl/startup-eso.idl ]; then
        export IDL_STARTUP=/scisoft/share/idl/startup-eso.idl
    fi

fi

#
#  Eclipse
#

if  [ -d /scisoft/share/eclipse ]; then
    export E_PATH=/scisoft/share/eclipse/eclipse
    export E_VERBOSE=1
    export E_MAXMEM=64
    export E_MAXSWAP=128
    export E_TMPDIR=.
    unset E_DEBUG
    if [ "X$PATH" != X ]; then
        export PATH=$PATH:$E_PATH/bin
    else
        export PATH=$E_PATH/bin
    fi
    if [ "X$MANPATH" != X ]; then
        export MANPATH=$MANPATH:$E_PATH/man
    else
        export MANPATH=$E_PATH/man
    fi
fi

#
#  Tiny Tim HST psf software
#

if [ -d /scisoft/share/tinytim ]; then
    export TINYTIM=/scisoft/share/tinytim
fi

#
#  PGPLOT graphics software
#

export PGPLOT_DIR=/scisoft/share/pgplot
export PGPLOT_FONT=/scisoft/share/pgplot/grfont.dat
export PGPLOT_DEV=/xwin

#
#  MIDAS
#

if [ -d /scisoft/lib/midas ]; then
    export MIDASHOME=/scisoft/lib/midas
    export MIDVERS=05SEPpl1.3
fi

#
#  XPA - disable warning at startup
#

export XPA_VERSIONCHECK=0

#
#  Ggobi
#

export GGOBI_HOME=/scisoft/lib/ggobi

#
#  Setup a search path for Latex to include astronomical journals
#

if [ "X$TEXINPUTS" != X ]; then
    export TEXINPUTS=$TEXINPUTS:`echo /scisoft/share/texmf/* | sed 's/ /:/g'`
else
    #  Note the leading ':' which means that latex should search the
    #  default paths.
    export TEXINPUTS=:`echo /scisoft/share/texmf/* | sed 's/ /:/g'`
fi

#
#  Python
#

if [ -d /scisoft/share/python ]; then
    if [ "X$PYTHONPATH" != X ]; then
        export PYTHONPATH=$HOME/PythonModules:$PYTHONPATH
    else
        export PYTHONPATH=$HOME/PythonModules
    fi
fi

#
#  DISLIN stuff
#

export DISLIN=/scisoft/share/dislin

#
#  Gildas
#

export GAG_PATH=/scisoft/share/pc-fedora3-ifort/etc

#
#  Check whether there is a /scilocal available and if so execute
#  its Setup.csh file
#

if [ -f /scilocal/bin/Setup.bash ]; then
    . /scilocal/bin/Setup.bash
fi
